import { REQUEST_BASE_URL } from '@/env';
import {
  userInfoRespSchema,
  userInfoRespSchemaNullable,
  type AnyRespSchema,
  type SucceedRespOf,
  type SucceedUserInfoResp,
  type SucceedUserInfoRespNullable,
} from '@/schemas/response';
import { useUserStore } from '@/stores/user';
import type { Nullable, Override } from '@/utils/types';
import axios, { type AxiosError, type AxiosRequestConfig, type Method } from 'axios';
import { ElMessage } from 'element-plus';
import type { z } from 'zod';

export const axiosInstance = axios.create({
  baseURL: REQUEST_BASE_URL,
});
// 自动添加token到请求中.
axiosInstance.interceptors.request.use((config) => {
  const userStore = useUserStore();
  config.headers.setAuthorization(userStore.token, false);
  return config;
});
// 自动获取响应中的token.
axiosInstance.interceptors.response.use((response) => {
  const userStore = useUserStore();
  const authorization = response.headers['set-authorization'] as string | undefined;
  if (authorization) {
    console.log(123);
    userStore.token = authorization;
  }
  return response;
});
export type RawResp = Record<string, unknown>;
const errorMessage = (description: string, code: string | number | undefined, msg: string) =>
  `${description}：(${code})${msg}`;

interface RequestOptions extends Override<AxiosRequestConfig, { method?: Method }, 'url'> {
  errorDescription?: string;
}
export function parseRawResp<T extends AnyRespSchema>(
  schema: T,
  raw: RawResp | undefined,
  errorDescription: string,
): SucceedRespOf<z.infer<T>> | undefined {
  if (!raw) return;
  const resp = schema.parse(raw);
  if (resp.type === 'error') {
    ElMessage.error(errorMessage(errorDescription, resp.code, resp.msg));
    return;
  }
  return resp as any;
}
export async function request<T extends AnyRespSchema>(
  url: string,
  schema: T,
  options: RequestOptions = {},
) {
  const { errorDescription = '请求错误' } = options;
  return parseRawResp(
    schema,
    await axiosInstance
      .request<RawResp>(Object.assign(options, { url }))
      .then((r) => r.data)
      .catch((err: AxiosError): undefined => {
        ElMessage.error(errorMessage(errorDescription, err.code, err.message));
      }),
    errorDescription,
  );
}
export async function getUserInfo(): Promise<SucceedUserInfoResp>;
export async function getUserInfo(userID: number): Promise<SucceedUserInfoRespNullable>;
export async function getUserInfo(userID?: number) {
  let url = '/api/user/info';
  if (userID !== undefined) {
    url += `/${userID}`;
  }
  const resp = await request(
    url,
    userID !== undefined ? userInfoRespSchemaNullable : userInfoRespSchema,
  );
  return resp;
}
export const getAvatarURL = (avatar: Nullable<string>) =>
  (avatar && `${import.meta.env.VITE_REQUEST_BASE_URL}/api/user/avatar/${avatar}`) ?? undefined;
