import { z } from 'zod';

function createRespSchema<T extends z.ZodTypeAny>(data: T) {
  return z.union([
    z
      .object({
        code: z.number().min(200).max(299),
        msg: z.string(),
        time: z.coerce.date(),
      })
      .extend({ data: data })
      .transform((raw) =>
        Object.assign(raw, {
          type: 'success',
        } as const),
      ),
    z
      .object({
        code: z.number(),
        msg: z.string(),
        time: z.coerce.date(),
      })
      .transform((raw) =>
        Object.assign(raw, {
          type: 'error',
        } as const),
      ),
  ]);
}

export type SucceedRespOf<T> = Extract<T, { type: 'success' }>;
export type ErrorRespOf<T> = Exclude<SucceedRespOf<T>, T>;
export type AnyRespSchema = ReturnType<typeof createRespSchema>;
export const ordinarySchema = createRespSchema(z.literal(true));
export const loginRespSchema = ordinarySchema;
export const registerRespSchema = ordinarySchema;
export const idAndNameSchema = z.object({
  id: z.number(),
  name: z.string(),
});

const authSchema = idAndNameSchema.extend({
  permissions: idAndNameSchema.array(),
});
const userInfoDataSchema = idAndNameSchema.extend({
  avatar: z.nullable(z.string()),
  auth: authSchema,
  club: z.nullable(
    idAndNameSchema.extend({
      commit: z.string(),
      auth: authSchema,
    }),
  ),
});
export const userInfoRespSchema = createRespSchema(userInfoDataSchema);
export const userInfoRespSchemaNullable = createRespSchema(userInfoDataSchema.nullable());
export type SucceedUserInfoResp = SucceedRespOf<z.infer<typeof userInfoRespSchema>>;
export type SucceedUserInfoRespNullable = SucceedRespOf<z.infer<typeof userInfoRespSchemaNullable>>;
export type UserInfo = SucceedUserInfoResp['data'];
export const verifyRespSchema = createRespSchema(
  z
    .object({
      img: z.string(),
      key: z.string(),
    })
    .transform((raw) => {
      raw.img = `data:image/png;base64,${raw.img}`;
      return raw;
    }),
);
export const uploadAvatarRespSchema = createRespSchema(z.string());
